@@ -257,9 +257,14 @@ public class Index {
         }
     }
 
-    Node getRoot(Session session) {
+    int getRoot() {
+        return (root == null) ? -1
+                              : root.getKey();
+    }
 
-        if (isTemp) {
+    private Node getRoot(Session session) {
+
+        if (isTemp && session != null) {
             return session.getIndexRoot(indexName, onCommitPreserve);
         } else {
             return root;
@@ -289,7 +294,7 @@ public class Index {
                 break;
             }
 
-            compare = compareRowUnique(session, row, n.getRow());
+            compare = compareRowForInsert(session, row, n.getRow());
 
             if (compare == 0) {
                 throw Trace.error(Trace.VIOLATION_OF_UNIQUE_INDEX,
@@ -314,6 +319,8 @@ public class Index {
             int sign = isleft ? 1
                               : -1;
 
+            x = x.getUpdatedNode();
+
             switch (x.getBalance() * sign) {
 
                 case 1 :
@@ -332,30 +339,38 @@ public class Index {
                         replace(session, x, l);
                         set(x, isleft, child(l, !isleft));
                         set(l, !isleft, x);
+
+                        x = x.getUpdatedNode();
+
                         x.setBalance(0);
+
+                        l = l.getUpdatedNode();
+
                         l.setBalance(0);
                     } else {
                         Node r = child(l, !isleft);
 
                         replace(session, x, r);
-                        set(l, !isleft, child(r, isleft));
+                        set(l, !isleft, child(r.getUpdatedNode(), isleft));
                         set(r, isleft, l);
-                        set(x, isleft, child(r, !isleft));
+                        set(x, isleft, child(r.getUpdatedNode(), !isleft));
                         set(r, !isleft, x);
 
-                        int rb = r.getBalance();
+                        int rb = r.getUpdatedNode().getBalance();
 
-                        x.setBalance((rb == -sign) ? sign
-                                                   : 0);
-                        l.setBalance((rb == sign) ? -sign
-                                                  : 0);
-                        r.setBalance(0);
+                        x.getUpdatedNode().setBalance((rb == -sign) ? sign
+                                                                    : 0);
+                        l.getUpdatedNode().setBalance((rb == sign) ? -sign
+                                                                   : 0);
+                        r.getUpdatedNode().setBalance(0);
                     }
 
                     return;
             }
 
-            if (x.equals(getRoot(session))) {
+            x = x.getUpdatedNode();
+
+            if (x.isRoot()) {
                 return;
             }
 
@@ -410,57 +425,93 @@ public class Index {
             // swap d and x
             int b = x.getBalance();
 
+            x = x.getUpdatedNode();
+
             x.setBalance(d.getBalance());
+
+            d = d.getUpdatedNode();
+
             d.setBalance(b);
 
             // set x.parent
             Node xp = x.getParent();
             Node dp = d.getParent();
 
-            if (d == getRoot(session)) {
+            x = x.getUpdatedNode();
+
+            if (d.isRoot()) {
                 setRoot(session, x);
             }
 
             x.setParent(dp);
 
             if (dp != null) {
-                if (dp.getRight().equals(d)) {
+                dp = dp.getUpdatedNode();
+
+                if (dp.isRight(d)) {
                     dp.setRight(x);
                 } else {
                     dp.setLeft(x);
                 }
             }
 
-            // for in-memory tables we could use: d.rData=x.rData;
-            // but not for cached tables
             // relink d.parent, x.left, x.right
-            if (xp == d) {
+            d = d.getUpdatedNode();
+
+            if (d.equals(xp)) {
                 d.setParent(x);
 
-                if (d.getLeft().equals(x)) {
+                if (d.isLeft(x)) {
+                    x = x.getUpdatedNode();
+
                     x.setLeft(d);
-                    x.setRight(d.getRight());
+
+                    Node dr = d.getRight();
+
+                    x = x.getUpdatedNode();
+
+                    x.setRight(dr);
                 } else {
                     x.setRight(d);
-                    x.setLeft(d.getLeft());
+
+                    Node dl = d.getLeft();
+
+                    x = x.getUpdatedNode();
+
+                    x.setLeft(dl);
                 }
             } else {
                 d.setParent(xp);
+
+                xp = xp.getUpdatedNode();
+
                 xp.setRight(d);
-                x.setRight(d.getRight());
-                x.setLeft(d.getLeft());
+
+                Node dl = d.getLeft();
+                Node dr = d.getRight();
+
+                x = x.getUpdatedNode();
+
+                x.setLeft(dl);
+                x.setRight(dr);
             }
 
             x.getRight().setParent(x);
             x.getLeft().setParent(x);
 
             // set d.left, d.right
+            d = d.getUpdatedNode();
+
             d.setLeft(n);
 
             if (n != null) {
+                n = n.getUpdatedNode();
+
                 n.setParent(d);
             }
 
+            d = d.getUpdatedNode();
+
             d.setRight(null);
 
             x = d;
@@ -471,6 +522,7 @@ public class Index {
         replace(session, x, n);
 
         n = x.getParent();
+        x = x.getUpdatedNode();
 
         x.delete();
 
@@ -480,6 +532,8 @@ public class Index {
             int sign = isleft ? 1
                               : -1;
 
+            x = x.getUpdatedNode();
+
             switch (x.getBalance() * sign) {
 
                 case -1 :
@@ -501,13 +555,23 @@ public class Index {
                         set(r, isleft, x);
 
                         if (b == 0) {
+                            x = x.getUpdatedNode();
+
                             x.setBalance(sign);
+
+                            r = r.getUpdatedNode();
+
                             r.setBalance(-sign);
 
                             return;
                         }
 
+                        x = x.getUpdatedNode();
+
                         x.setBalance(0);
+
+                        r = r.getUpdatedNode();
+
                         r.setBalance(0);
 
                         x = r;
@@ -516,16 +580,26 @@ public class Index {
 
                         replace(session, x, l);
 
+                        l = l.getUpdatedNode();
                         b = l.getBalance();
 
                         set(r, isleft, child(l, !isleft));
                         set(l, !isleft, r);
                         set(x, !isleft, child(l, isleft));
                         set(l, isleft, x);
+
+                        x = x.getUpdatedNode();
+
                         x.setBalance((b == sign) ? -sign
                                                  : 0);
+
+                        r = r.getUpdatedNode();
+
                         r.setBalance((b == -sign) ? sign
                                                   : 0);
+
+                        l = l.getUpdatedNode();
+
                         l.setBalance(0);
 
                         x = l;
@@ -542,27 +616,26 @@ public class Index {
 
         Node node = findNotNull(session, rowdata, rowColMap, true);
 
-        return node == null ? emptyIterator
-                            : new IndexRowIterator(session, this, node);
+        return getIterator(session, node);
     }
 
     RowIterator findFirstRowForDelete(Session session, Object[] rowdata,
                                       int[] rowColMap) throws HsqlException {
 
-        Node node = findNotNull(session, rowdata, rowColMap, true);
+        Node node           = findNotNull(session, rowdata, rowColMap, true);
+        IndexRowIterator it = getIterator(session, node);
 
-        if (node == null) {
-            return emptyIterator;
-        } else {
-            RowIterator it = new IndexRowIterator(session, this, node);
-
-            updatableIterators.link((IndexRowIterator) it);
-
-            return it;
+        if (node != null) {
+            updatableIterators.link(it);
         }
+
+        return it;
     }
 
-    public Row findRow(Session session, Row row) throws HsqlException {
+    /**
+     * Finds an existing row
+     */
+    Row findRow(Session session, Row row) throws HsqlException {
 
         Node node = search(session, row);
 
@@ -702,7 +775,7 @@ public class Index {
      * only. The rowdata has the same column mapping as this table.
      *
      * @param rowdata array containing table row data
-     * @return matching node
+     * @return iterator
      * @throws HsqlException
      */
     RowIterator findFirstRow(Session session,
@@ -731,8 +804,7 @@ public class Index {
             }
         }
 
-        return found == null ? emptyIterator
-                             : new IndexRowIterator(session, this, found);
+        return getIterator(session, found);
     }
 
     /**
@@ -742,7 +814,7 @@ public class Index {
      * @param value value to match
      * @param compare comparison Expression type
      *
-     * @return matching node
+     * @return iterator
      *
      * @throws HsqlException
      */
@@ -826,14 +898,13 @@ public class Index {
             }
         }
 
-        return x == null ? emptyIterator
-                         : new IndexRowIterator(session, this, x);
+        return getIterator(session, x);
     }
 
     /**
      * Finds the first node where the data is not null.
      *
-     * @return matching node
+     * @return iterator
      *
      * @throws HsqlException
      */
@@ -874,8 +945,7 @@ public class Index {
             }
         }
 
-        return x == null ? emptyIterator
-                         : new IndexRowIterator(session, this, x);
+        return getIterator(session, x);
     }
 
     /**
@@ -899,8 +969,7 @@ public class Index {
             depth++;
         }
 
-        return x == null ? emptyIterator
-                         : new IndexRowIterator(session, this, x);
+        return getIterator(session, x);
     }
 
     /**
@@ -992,12 +1061,14 @@ public class Index {
     private void replace(Session session, Node x,
                          Node n) throws HsqlException {
 
-        if (x.equals(getRoot(session))) {
-            setRoot(session, n);
-
+        if (x.isRoot()) {
             if (n != null) {
+                n = n.getUpdatedNode();
+
                 n.setParent(null);
             }
+
+            setRoot(session, n);
         } else {
             set(x.getParent(), x.isFromLeft(), n);
         }
@@ -1014,6 +1085,8 @@ public class Index {
      */
     private void set(Node x, boolean isleft, Node n) throws HsqlException {
 
+        x = x.getUpdatedNode();
+
         if (isleft) {
             x.setLeft(n);
         } else {
@@ -1021,6 +1094,8 @@ public class Index {
         }
 
         if (n != null) {
+            n = n.getUpdatedNode();
+
             n.setParent(x);
         }
     }
@@ -1034,13 +1109,13 @@ public class Index {
      *
      * @throws HsqlException
      */
-    Node search(Session session, Row row) throws HsqlException {
+    private Node search(Session session, Row row) throws HsqlException {
 
         Object[] d = row.getData();
         Node     x = getRoot(session);
 
         while (x != null) {
-            int c = compareRowUnique(session, row, x.getRow());
+            int c = compareRowForInsert(session, row, x.getRow());
 
             if (c == 0) {
                 return x;
@@ -1058,8 +1133,7 @@ public class Index {
      * Compares two table rows based on the columns of this index. The rowColMap
      * parameter specifies which columns of the other table are to be compared
      * with the colIndex columns of this index. The rowColMap can cover all
-     * or only some columns of this index. The invisible column is never
-     * compared
+     * or only some columns of this index.
      *
      * @param a row from another table
      * @param rowColMap column indexes in the other table
@@ -1129,11 +1203,11 @@ public class Index {
      *
      * @throws HsqlException
      */
-    private int compareRowUnique(Session session, Row left,
-                                 Row right) throws HsqlException {
+    private int compareRowForInsert(Session session, Row newRow,
+                                    Row existingRow) throws HsqlException {
 
-        Object[] a       = left.getData();
-        Object[] b       = right.getData();
+        Object[] a       = newRow.getData();
+        Object[] b       = existingRow.getData();
         int      j       = 0;
         boolean  hasNull = false;
 
@@ -1166,7 +1240,7 @@ public class Index {
         }
 
         if (useRowId) {
-            int difference = left.getPos() - right.getPos();
+            int difference = newRow.getPos() - existingRow.getPos();
 
             if (difference < 0) {
                 difference = -1;
@@ -1211,6 +1285,17 @@ public class Index {
         }
     }
 
+    private IndexRowIterator getIterator(Session session, Node x) {
+
+        if (x == null) {
+            return emptyIterator;
+        } else {
+            IndexRowIterator it = new IndexRowIterator(session, this, x);
+
+            return it;
+        }
+    }
+
     static class IndexRowIterator implements RowIterator {
 
         Session                    session;
@@ -1247,18 +1332,27 @@ public class Index {
 
                     return row;
                 } catch (Exception e) {
-                    throw new NoSuchElementException();
+                    throw new NoSuchElementException(e.getMessage());
                 }
             } else {
                 return null;
             }
         }
 
+        void updateForDelete(Node node) {
+
+            try {
+                if (node.equals(nextnode)) {
+                    nextnode = index.next(node);
+                }
+            } catch (Exception e) {}
+        }
+
         void link(IndexRowIterator other) {
 
             other.next = next;
-            next.last  = other;
             other.last = this;
+            next.last  = other;
             next       = other;
         }
 
@@ -1272,14 +1366,5 @@ public class Index {
                 next.last = last;
             }
         }
-
-        void updateForDelete(Node node) {
-
-            try {
-                if (node.equals(nextnode)) {
-                    nextnode = index.next(node);
-                }
-            } catch (Exception e) {}
-        }
     }
 }
