@@ -89,10 +89,8 @@ import org.hsqldb.Types;
  * However, in addition to this technique requiring explicit casts to the
  * org.hsqldb.jdbcXXX classes, some of these method calls require
  * <code>int</code> values that are defined only in the JDBC 2 or greater
- * version of the
- * <a href="http://java.sun.com/j2se/1.4/docs/api/java/sql/ResultSet.html">
- * <code>ResultSet</code></a> interface.  For this reason these values are
- * defined in {@link jdbcResultSet jdbcResultSet}.<p>
+ * version of the {@link java.sql.ResultSet ResultSet} interface.  For this
+ * reason these values are defined in {@link jdbcResultSet jdbcResultSet}.<p>
  *
  * In a JRE 1.1.x environment, calling JDBC 2 methods that take or return the
  * JDBC2-only <code>ResultSet</code> values can be achieved by referring
@@ -757,7 +755,7 @@ public class jdbcStatement implements Statement {
         checkClosed();
 
         if (direction != jdbcResultSet.FETCH_FORWARD) {
-            throw Util.notSupported;
+            throw Util.notSupported();
         }
     }
 
@@ -1166,7 +1164,7 @@ public class jdbcStatement implements Statement {
      */
 //#ifdef JDBC3
     public boolean getMoreResults(int current) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1197,7 +1195,7 @@ public class jdbcStatement implements Statement {
      */
 //#ifdef JDBC3
     public ResultSet getGeneratedKeys() throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1240,7 +1238,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public int executeUpdate(String sql,
                              int autoGeneratedKeys) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1279,7 +1277,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public int executeUpdate(String sql,
                              int[] columnIndexes) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1316,7 +1314,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public int executeUpdate(String sql,
                              String[] columnNames) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1372,7 +1370,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public boolean execute(String sql,
                            int autoGeneratedKeys) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1427,7 +1425,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public boolean execute(String sql,
                            int[] columnIndexes) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
@@ -1483,7 +1481,7 @@ public class jdbcStatement implements Statement {
 //#ifdef JDBC3
     public boolean execute(String sql,
                            String[] columnNames) throws SQLException {
-        throw Util.notSupported;
+        throw Util.notSupported();
     }
 
 //#endif JDBC3
