@@ -38,6 +38,7 @@ import org.hsqldb.DatabaseURL;
 import org.hsqldb.HsqlException;
 import org.hsqldb.Trace;
 import org.hsqldb.lib.Set;
+import org.hsqldb.lib.SimpleLog;
 import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.java.JavaSystem;
 
@@ -70,6 +71,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final String VERSION_STRING_1_8_0     = "1.8.0";
     public static final String FIRST_COMPATIBLE_VERSION = "1.8.0";
     public static final String THIS_VERSION             = "1.8.0";
+    public static final String THIS_FULL_VERSION        = "1.8.0.4";
     public static final String PRODUCT_NAME = "HSQL Database Engine";
     public static final int    MAJOR                    = 1,
                                MINOR                    = 8,
@@ -177,9 +179,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     public HsqlDatabaseProperties(Database db) {
 
-        super(db.getPath(), db.isStoredFileAccess() ? db.getFileAccess()
-                                                    : null, db
-                                                    .isFilesInJar());
+        super(db.getPath(), db.getFileAccess(), db.isFilesInJar());
 
         database = db;
 
@@ -300,7 +300,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
         boolean exists;
 
-        if (database.getType().equals(DatabaseURL.S_MEM)) {
+        if (!DatabaseURL.isFileBasedDatabaseType(database.getType())) {
             return true;
         }
 
@@ -366,14 +366,19 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     public void save() throws HsqlException {
 
-        if (database.getType().equals(DatabaseURL.S_MEM)
+        if (!DatabaseURL.isFileBasedDatabaseType(database.getType())
                 || database.isFilesReadOnly() || database.isFilesInJar()) {
             return;
         }
 
         try {
-            super.save();
+            super.save(fileName + ".properties" + ".new");
+            fa.renameElement(fileName + ".properties" + ".new",
+                             fileName + ".properties");
         } catch (Exception e) {
+            database.logger.appLog.logContext(SimpleLog.LOG_ERROR,
+                                              "failed");
+
             throw Trace.error(Trace.FILE_IO_ERROR,
                               Trace.LOAD_SAVE_PROPERTIES, new Object[] {
                 fileName, e
@@ -461,14 +466,19 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                                              : 20000;
     }
 
+    static String MODIFIED_NO = "no";
+    static String MODIFIED_YES = "yes";
+    static String MODIFIED_NEW = "no-new-files";
+
+
     public void setDBModified(int mode) throws HsqlException {
 
-        String value = "no";
+        String value = MODIFIED_NO;
 
         if (mode == FILES_MODIFIED) {
-            value = "yes";
+            value = MODIFIED_YES;
         } else if (mode == FILES_NEW) {
-            value = "no-new-files";
+            value = MODIFIED_NEW;
         }
 
         setProperty(db_modified, value);
@@ -479,9 +489,9 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
         String value = getProperty("modified");
 
-        if ("yes".equals(value)) {
+        if (MODIFIED_YES.equals(value)) {
             return FILES_MODIFIED;
-        } else if ("no-new-files".equals(value)) {
+        } else if (MODIFIED_NEW.equals(value)) {
             return FILES_NEW;
         }
 
