@@ -78,6 +78,10 @@ public class Logger {
     boolean  logStatements;
     boolean  syncFile = false;
 
+    public Logger() {
+        appLog = new SimpleLog(null, SimpleLog.LOG_NONE, false);
+    }
+
     /**
      *  Opens the specified Database object's database files and starts up
      *  the logging process. <p>
@@ -95,12 +99,12 @@ public class Logger {
         int loglevel = db.getProperties().getIntegerProperty(
             HsqlDatabaseProperties.hsqldb_applog, 0);
 
-        appLog = new SimpleLog(path + ".app.log", loglevel,
-                               !db.isFilesReadOnly());
+        if (loglevel != SimpleLog.LOG_NONE) {
+            appLog = new SimpleLog(path + ".app.log", loglevel,
+                                   !db.isFilesReadOnly());
+        }
 
-        appLog.sendLine("Database (re)opened: "
-                        + db.getFileAccess().getClass().getName() + " "
-                        + HsqlDateTime.getSytemTimeString());
+        appLog.sendLine(SimpleLog.LOG_ERROR, "Database (re)opened");
 
         logStatements = false;
 
@@ -142,8 +146,7 @@ public class Logger {
 
         if (log == null) {
             if (appLog != null) {
-                appLog.sendLine("Database closed: "
-                                + HsqlDateTime.getSytemTimeString());
+                appLog.sendLine(SimpleLog.LOG_ERROR, "Database closed");
                 appLog.close();
             }
 
@@ -168,8 +171,8 @@ public class Logger {
             }
         } catch (Throwable e) {
             if (appLog != null) {
-                appLog.sendLine("Database closed: "
-                                + HsqlDateTime.getSytemTimeString());
+                appLog.sendLine(SimpleLog.LOG_NORMAL, "error");
+                appLog.logContext(e);
                 appLog.close();
             }
 
@@ -179,8 +182,7 @@ public class Logger {
         }
 
         if (appLog != null) {
-            appLog.sendLine("Database closed: "
-                            + HsqlDateTime.getSytemTimeString());
+            appLog.sendLine(SimpleLog.LOG_ERROR, "Database closed");
             appLog.close();
         }
 
@@ -236,7 +238,8 @@ public class Logger {
      * @throws  HsqlException if there is a problem recording the Log
      *      entry
      */
-    public void logConnectUser(Session session) throws HsqlException {
+    public synchronized void logConnectUser(Session session)
+    throws HsqlException {
 
         if (logStatements) {
             writeToLog(session, session.getUser().getConnectStatement());
@@ -252,40 +255,41 @@ public class Logger {
      * @param  statement the SQL statement to Log
      * @throws  HsqlException if there is a problem recording the entry
      */
-    public void writeToLog(Session session,
-                           String statement) throws HsqlException {
+    public synchronized void writeToLog(Session session,
+                                        String statement)
+                                        throws HsqlException {
 
         if (logStatements && log != null) {
             log.writeStatement(session, statement);
         }
     }
 
-    public void writeInsertStatement(Session session, Table table,
-                                     Object[] row) throws HsqlException {
+    public synchronized void writeInsertStatement(Session session,
+            Table table, Object[] row) throws HsqlException {
 
         if (logStatements) {
             log.writeInsertStatement(session, table, row);
         }
     }
 
-    public void writeDeleteStatement(Session session, Table t,
-                                     Object[] row) throws HsqlException {
+    public synchronized void writeDeleteStatement(Session session, Table t,
+            Object[] row) throws HsqlException {
 
         if (logStatements) {
             log.writeDeleteStatement(session, t, row);
         }
     }
 
-    public void writeSequenceStatement(Session session,
-                                       NumberSequence s)
-                                       throws HsqlException {
+    public synchronized void writeSequenceStatement(Session session,
+            NumberSequence s) throws HsqlException {
 
         if (logStatements) {
             log.writeSequenceStatement(session, s);
         }
     }
 
-    public void writeCommitStatement(Session session) throws HsqlException {
+    public synchronized void writeCommitStatement(Session session)
+    throws HsqlException {
 
         if (logStatements) {
             log.writeCommitStatement(session);
@@ -296,14 +300,14 @@ public class Logger {
     /**
      * Called after commits or after each statement when autocommit is on
      */
-    public void synchLog() {
+    public synchronized void synchLog() {
 
         if (logStatements && syncFile) {
             log.synchLog();
         }
     }
 
-    public void synchLogForce() {
+    public synchronized void synchLogForce() {
 
         if (logStatements) {
             log.synchLog();
@@ -324,10 +328,12 @@ public class Logger {
      * @throws  HsqlException if there is a problem checkpointing the
      *      database
      */
-    public void checkpoint(boolean mode) throws HsqlException {
+    public synchronized void checkpoint(boolean mode) throws HsqlException {
 
         if (logStatements) {
+            appLog.logContext(appLog.LOG_NORMAL, "start");
             log.checkpoint(mode);
+            appLog.logContext(appLog.LOG_NORMAL, "end");
         }
     }
 
@@ -337,7 +343,7 @@ public class Logger {
      *
      * @param  megas size in MB
      */
-    public void setLogSize(int megas) {
+    public synchronized void setLogSize(int megas) {
 
         if (log != null) {
             log.setLogSize(megas);
@@ -350,7 +356,7 @@ public class Logger {
      *
      * @param  i The type
      */
-    public void setScriptType(int i) throws HsqlException {
+    public synchronized void setScriptType(int i) throws HsqlException {
 
         if (log != null) {
             log.setScriptType(i);
@@ -370,7 +376,7 @@ public class Logger {
      *
      * @param  delay in seconds
      */
-    public void setWriteDelay(int delay) {
+    public synchronized void setWriteDelay(int delay) {
 
         if (log != null) {
             syncFile = (delay == 0);
